package es.masterupv.android.worldcurrconvert;

import android.app.Activity;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.content.res.Resources;
import android.os.AsyncTask;
import android.preference.PreferenceManager;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.TextView;
import android.widget.Toast;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

import java.net.URL;
import java.util.Vector;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

public class Indices extends ListActivity {
    private TextView monedaLocal;
    private String[] listaMonedas;
    private int indiceMonedaLocal;
    private Activity actividad;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_indices);

        actividad=this;

        monedaLocal = (TextView) findViewById(R.id.tOrigen);

        Resources res = this.getResources();
        listaMonedas = res.getStringArray(R.array.monedaLocal);

        SharedPreferences preferencias = PreferenceManager.getDefaultSharedPreferences(this);
        indiceMonedaLocal = Integer.parseInt(preferencias.getString("monedaLocal", "42"));
        monedaLocal.setText(listaMonedas[indiceMonedaLocal]);

        CalcularConversion tarea = new CalcularConversion();
        tarea.execute(indiceMonedaLocal);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_indices, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();

        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }


    private class CalcularConversion extends AsyncTask<Integer,Integer,String>
    {
        private ProgressDialog progreso;
        private int indiceMoneda;

        @Override
        protected void onPreExecute() {
            progreso = new ProgressDialog(Indices.this);
            progreso.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
            progreso.setMessage("Obteniendo indices...");
            progreso.setCancelable(true);
            progreso.setOnCancelListener(new DialogInterface.OnCancelListener() { //Añadir un escuchador de evento
                @Override
                public void onCancel(DialogInterface dialog) {
                    CalcularConversion.this.cancel(true);
                }
            });
            progreso.setMax(100);
            progreso.setProgress(0);
            progreso.show();
        }

        @Override
        protected String doInBackground(Integer... n) {
            indiceMoneda = n[0];

            StringBuilder resultado = new StringBuilder();
            String informacion="";

            for (int i=0;i<listaMonedas.length;i++) {
                if (i!=indiceMoneda) {
                    informacion=obtenerMonedaIndice(indiceMoneda,i);
                    if (!informacion.equals("")) {
                        resultado.append(obtenerMonedaIndice(indiceMoneda, i));
                        if (i != listaMonedas.length - 1) resultado.append(",");
                    }
                    publishProgress(i * 100 / listaMonedas.length-1);
                }
            }
            return resultado.toString();
        }

        @Override
        protected void onProgressUpdate(Integer... porc) {
            progreso.setProgress(porc[0]);
        }

        @Override
        protected void onPostExecute(String res) {
            Vector<String> resultados = new Vector<String>();
            String[] auxiliar;
            if (res.length()>0) {
                auxiliar = res.split(",");
                for (int i=0;i<auxiliar.length;i++) resultados.add(auxiliar[i]);
                setListAdapter(new IndicesAdaptador(actividad,resultados));
            }
            progreso.dismiss();
        }

        private String obtenerMonedaIndice(int indOrigen, int indDestino) {
            String resultado = "";
            try {
                URL url = new URL("http://www.webservicex.net/CurrencyConvertor.asmx/ConversionRate?FromCurrency=" + listaMonedas[indOrigen].substring(0,3) + "&ToCurrency=" + listaMonedas[indDestino].substring(0,3));

                SAXParserFactory factoria = SAXParserFactory.newInstance();
                SAXParser analizador = factoria.newSAXParser();
                XMLReader lector = analizador.getXMLReader();

                InterpreteXML interpreteXML = new InterpreteXML();

                lector.setContentHandler(interpreteXML);

                lector.parse(new InputSource(url.openStream()));

                if (interpreteXML.getFactorConversion()>=0) resultado = String.valueOf(interpreteXML.getFactorConversion()) + " " + ((String) listaMonedas[indDestino]).substring(0,3) + " ("+((String) listaMonedas[indDestino]).substring(4)+")";

            } catch (Exception ex) {
                Toast.makeText(actividad.getApplicationContext(), "ERROR: " + getResources().getString(R.string.error_red), Toast.LENGTH_LONG).show();
            }
            return resultado;
        }

        private class InterpreteXML extends DefaultHandler {
            private double factorConversion=0.0;
            private StringBuilder contenido = new StringBuilder();

            public Double getFactorConversion() {
                return factorConversion;
            }

            @Override
            public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
                super.startElement(uri, localName, qName, attributes);
                contenido.setLength(0);
            }

            @Override
            public void characters(char[] ch, int start, int length) throws SAXException {
                super.characters(ch, start, length);
                contenido.append(ch,start,length);
            }

            @Override
            public void endElement(String uri, String localName, String qName) throws SAXException {
                super.endElement(uri, localName, qName);
                if (localName.equals("double"))
                    factorConversion = Double.parseDouble(contenido.toString());
            }
        }

    }
}
